import os
from os import path
from pathlib import Path
import sys
import pycapo

from .logging import _LOG

_CAPO_PROPERTIES_ROOT = '/home/casa/capo'


def profile_file_in_path(profile, properties_path):
    """
    Check if the profile name provided is a valid name for a capo property file, by checking
    to see if a <profile>.properties file can be found in the path provided
    :param profile: the profile name we want to look for
    :param properties_path: a path to a directory where we expect to find capo property files
    :return: true if the profile given has a corresponding <profile>.properties file in the
    path provided.
    """
    valid_profile = False
    profile_names = [prop.split('.properties')[0] for prop in os.listdir(properties_path)
                     if prop.endswith('.properties')]
    if profile in profile_names:
        valid_profile = True
    return valid_profile


def get_my_capo_config(**kwargs):
    r"""
    A function to return a CapoConfig for the profile either passed in, found in the environment
    or derived from the location we've installed pyat.  We do a rudamentary test of the profile
    against a series of known capo properties directories (CAPO_PATH, /home/casa/capo, or
    $HOME/.capo) and print a warning if the profile is not found as a valid properties file in any
    of those location.  This is solely to augment the cryptic exception messages that might be
    thrown when trying to access a capo key from a file that doesn't exist.
    :param kwargs: may or may not contain a 'profile'
    :return: a CapoConfig object for the profile derived from the kwargs, env, or deployment
    location
    """
    if 'profile' in kwargs and kwargs['profile']:
        profile = kwargs['profile']
    elif 'CAPO_PROFILE' in os.environ:
        profile = os.environ['CAPO_PROFILE']
    else:
        # try to synthesize a profile from our installation root
        # if you don't set the profile and end up here, but you're running locally... this might
        # result in a profile of "<your virtualenv>", it's best to actually set it.
        path_breakdown = os.path.abspath(sys.argv[0]).split(os.path.sep)
        # CV sticks an extra subdirectory into the installation
        if path_breakdown[-3] == 'current':
            profile = path_breakdown[-4]
        else:
            profile = path_breakdown[-3]

    # We should have a profile at this point, but it might not be valid (if the user didn't provide
    # one), so now we'll try checking to see if that profile matchies a <profile>.properties file
    # in one of our known properties directories.  We'll check for a CAPO_PATH, then,
    # /home/casa/capo, then $HOME/.capo, and, if all that fails, we'll print a warning to the
    # console to augment any exception that might be thrown by trying to access a property from a
    # properties files that doesn't exist.
    profile_exists = False
    if 'CAPO_PATH' in os.environ:
        profile_exists = profile_file_in_path(profile, os.environ['CAPO_PATH'])
    elif path.exists(_CAPO_PROPERTIES_ROOT):
        profile_exists = profile_file_in_path(profile, _CAPO_PROPERTIES_ROOT)
    elif path.exists(str(Path.home()) + '/.capo'):
        profile_exists = profile_file_in_path(profile, str(Path.home()) + '/.capo')

    if not profile_exists:
        _LOG.error(f'The capo profile "{profile}" does not appear to match any known profile '
                   f'names.  This might result in unexpected behavior if the application attempts '
                   f'to access a property from a file that does not exist.')

    return pycapo.CapoConfig(profile=profile)
